﻿using KlerksSoft;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;

namespace EncodingDetector
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Read all encodings from the system and write file for each encoding.");

            string exampleStringToWrite = "üöäß;ÜÖÄß@€µ";

            List<string> savedFiles = new List<string>();
            ProcessModule processModule = Process.GetCurrentProcess().MainModule;
            string exePath = string.Empty;
            if (processModule != null)
            {
                exePath = Path.GetDirectoryName(processModule.FileName);
            }

            foreach (EncodingInfo encodingInfo in Encoding.GetEncodings())
            {
                string fileName = $"{encodingInfo.DisplayName}.txt";
                foreach (char c in Path.GetInvalidFileNameChars())
                {
                    fileName = fileName.Replace(c, '_');
                }

                fileName = Path.Combine(exePath ?? string.Empty, fileName);

                using (StreamWriter sw = new StreamWriter(File.Open(fileName, FileMode.Create), encodingInfo.GetEncoding()))
                {
                    sw.WriteLine(exampleStringToWrite);
                    savedFiles.Add(fileName);
                    Console.WriteLine($"File {fileName} saved.");
                }
            }

            string windows1252File = Path.Combine(exePath ?? string.Empty, "windows1252.txt");
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
            using (StreamWriter sw = new StreamWriter(File.Open(windows1252File, FileMode.Create), Encoding.GetEncoding(1252)))
            {
                sw.WriteLine(exampleStringToWrite);
                Console.WriteLine($"File {windows1252File} saved.");
            }


            Console.WriteLine("Press any key to detect encodings");
            Console.ReadKey();

            foreach (string savedFile in savedFiles)
            {
                DisplayEncodingInConsole(savedFile);
            }

            DisplayEncodingInConsole(windows1252File);

            Console.WriteLine("Finished");
            Console.ReadKey();
        }

        private static void DisplayEncodingInConsole(string fileName)
        {
            Encoding encoding = TextFileEncodingDetector.DetectTextFileEncoding(fileName);

            if (encoding is null)
            {
                Console.WriteLine($"File {fileName} is most probably encoded as {Encoding.Default}");
            }
            else
            {
                Console.WriteLine($"File {fileName} encoded as {encoding}");
            }
        }
    }
}